
import os
import tensorflow as tf

"""
model2savedmodel.py

Basic example script to 
  A) load a trained tf2 classification model (here a tf.keras.Model instance), 
  B) add image decoding and resizing layers to the input,
  C) embed the class names, to include in outputs,
  D) wrap it with a 'classification' signature describing input and outputs, 
  E) and write it to a SavedModel ready for serving with https://www.tensorflow.org/tfx/guide/serving. 

Note: tf models may be written to SavedModels in several different ways. Feel free to follow another path - but please see and replicate the decoding and resizing layers added by this example.

Author: msa@eiva.com 2021-09-12
"""

def model2savedmodel(
    kerasModel, classes=["Class1", "Class2", "Class3"], input_shape=[288, 512], savedmodel_dir="example_savedmodel_dir", savedmodel_version=1
):
    if input_shape[0] > input_shape[1]:
        print(
            "[Warning] Height of input images exceed the width. \
                That is probably not, what you want!"
        )
    output_path = os.path.join(savedmodel_dir, str(savedmodel_version))

    class ModelModule(tf.Module):
        def __init__(self, model, classes):
            self.model = model
            self.classes = tf.constant(classes)

        @tf.function(input_signature=[tf.TensorSpec(shape=(None), dtype=tf.string)])
        def classification(self, images_encoded):
            # decode
            images_decoded = tf.map_fn(
                self.decode, images_encoded, dtype=tf.uint8, name="images_decoded"
            )
            # resize
            images_resized = tf.image.resize(
                images_decoded,
                (input_shape[0], input_shape[1]),
                method=tf.image.ResizeMethod.BILINEAR,
            )

            # If your model need any other preprocessing not defined in kerasModel, do it now...

            class_prob, _ = self.model((images_resized, False)) # Your kerasModel arguments and outputs can be different!
            
            # If your model need any postprocessing not defined in your kerasModel, do it now...
            
            classes_repeated_for_batch = tf.tile(tf.reshape(self.classes, [1, -1]), [images_encoded.shape[0], 1])
            return {
                "classes": classes_repeated_for_batch
                "probabilities": class_prob,
            }

        @staticmethod
        def decode(image_buffer):
            """Decode JPEG/PNG/GIF encoded byte string"""
            image = tf.io.decode_image(image_buffer, channels=input_shape[2])
            image.set_shape([None, None, int(input_shape[2])])
            return image


    signatures = {
        "classification": module.classification,
        # "segmentation": module.segmentation,
        # "objectdetection": module.objectdetection
    }
    module = ModelModule(kerasModel, classes)
    tf.saved_model.save(module, output_path, signatures=signatures)
